searchNodes=[{"type":"module","title":"defarg","doc":"Erlang parse transform for permitting default arguments in functions Presently the Erlang syntax doesn't allow function arguments to have default parameters. Consequently a developer needs to replicate the function definition multiple times passing constant defaults to some parameters of functions. This parse transform addresses this shortcoming by extending the syntax of function definitions at the top level in a module to have a default expression such that for A / Default argument the Default will be used if the function is called in code without that argument. - export ( [ t / 2 ] ) . test ( A / 10 , B / 20 ) -&gt; A + B . The code above is transformed to: - export ( [ t / 2 ] ) . - export ( [ t / 0 , t / 1 ] ) . test ( ) -&gt; test ( 10 ) ; test ( A ) -&gt; test ( A , 20 ) ; test ( A , B ) -&gt; A + B . The arguments with default values must be at the end of the argument list: test ( A , B , C / 1 ) -&gt; %% This is valid . . . test ( A / 1 , B , C ) -&gt; %% This is invalid . . . Default arguments must be constants or arithmetic expressions. Function calls are not supported as default arguments due to the limitations of the Erlang parser.","ref":"defarg.html"},{"type":"function","title":"defarg.parse_transform/2","doc":"parse_transform entry point","ref":"defarg.html#parse_transform/2"},{"type":"module","title":"erlpipe","doc":"Erlang pipeline parse transform This transform implements a parser syntax extension that enables application of cascading function calls using the / operator. In the LHS / RHS / ... Last. notation, the result of evaluation of the LHS expression is passed as an argument to the RHS expression. This process continues until the Last expression is evaluated. The head element of the pipeline must be either a term to which the arithmetic division /` operator cannot apply (i.e. not integers, floats, functions), or if you need to pass integer(s) or float(s), wrap them in a list brackets. This transfor is inspired by the similar functionality in Linux (i.e. `| pipe) and Elixir ( |&gt; pipe). When using this as a parse transform, include the {parse_transform,erlpipe} compiler option. The following examples illustrate the work of the transform, in which: test1 ( A ) -&gt; [ A ] / fun1 / mod : fun2 / fun3 . test2 ( A , B ) -&gt; [ A , B ] / fun4 / fun5 ( ) / io : format ( &quot; ~p \\n&quot; , [ _ ] ) . will be transformed to: test1 ( A ) -&gt; fun3 ( mod : fun2 ( fun1 ( A ) ) ) . test2 ( A , B ) -&gt; io : format ( &quot; ~p \\n&quot; , [ fun5 ( fun4 ( A , B ) ) ] ) . Similarly to Elixir, a special tap/2 function is implemented, which passes the given argument to an anonymous function, returning the argument itself. The following: f ( A ) -&gt; A + 1 . . . . test_tap ( ) -&gt; [ 10 ] / tap ( f ) / tap ( fun f / 1 ) / tap ( fun ( I ) -&gt; I + 1 end ) . is equivalent to: . . . test_tap ( ) -&gt; begin f ( 10 ) , begin f ( 10 ) , begin ( fun ( I ) -&gt; I end ) ( 10 ) 10 end end end . For debugging the AST of the resulting transform, pass the following options to the erlc compiler: -Derlpipe_orig - print the original AST before the transform -Derlpipe_ast - print the transformed AST -Derlpipe_src - print the resulting source code after the transform","ref":"erlpipe.html"},{"type":"function","title":"erlpipe.parse_transform/2","doc":"parse_transform entry point","ref":"erlpipe.html#parse_transform/2"},{"type":"module","title":"etran","doc":"Apply all transforms in the etran application","ref":"etran.html"},{"type":"function","title":"etran.parse_transform/2","doc":"parse_transform entry point","ref":"etran.html#parse_transform/2"},{"type":"module","title":"etran_util","doc":"Erlang parse transform utility functions","ref":"etran_util.html"},{"type":"function","title":"etran_util.apply_transform/4","doc":"Apply parse transform with debug printing options","ref":"etran_util.html#apply_transform/4"},{"type":"function","title":"etran_util.debug_options/2","doc":"Get parse transforms debug options","ref":"etran_util.html#debug_options/2"},{"type":"function","title":"etran_util.parse_options/2","doc":"Check if KnownFlags are found in Options.","ref":"etran_util.html#parse_options/2"},{"type":"function","title":"etran_util.process/4","doc":"Call Fun for the AST and optionally print debug info","ref":"etran_util.html#process/4"},{"type":"function","title":"etran_util.source_forms/2","doc":"Decompile source code from the AST","ref":"etran_util.html#source_forms/2"},{"type":"function","title":"etran_util.transform/2","doc":"Transform Forms by applying a lambda Fun .","ref":"etran_util.html#transform/2"},{"type":"function","title":"etran_util.transform/3","doc":"Transform Forms by applying a lambda Fun .","ref":"etran_util.html#transform/3"},{"type":"module","title":"iif","doc":"Conditional expression functions This module exports a parse transform and implements several condition-checking functions. When using this as a parse transform, include the {parse_transform,iif} compiler option. In this case for given expressions A , B , C , and D the following code transforms will be done: iif ( A , B ) -&gt; case A of true -&gt; B ; _ -&gt; undefined end iif ( A , B , C ) -&gt; case A of true -&gt; B ; _ -&gt; C end iif ( A , B , C , D ) -&gt; case A of B -&gt; C ; _ -&gt; D end nvl ( A , B ) -&gt; case A of false -&gt; B ; undefined -&gt; B ; [ ] -&gt; B ; _ -&gt; A end nvl ( A , B , C ) -&gt; case A of false -&gt; B ; undefined -&gt; B ; [ ] -&gt; B ; _ -&gt; C end For debugging the AST of the resulting transform, use iif_debug command-line option: erlc - Diif_debug = 1 . . . % Prints AST before the transform erlc - Diif_debug = 2 . . . % Prints AST after the transform erlc - Diif_debug [ = 3 ] . . . % Prints AST before/after the transform Alternative to using this module as a parse_transform, it implements several iif/3,4 exported functions that can be used without the transform.","ref":"iif.html"},{"type":"function","title":"iif.format_ne/3","doc":"Format if first argument is not empty","ref":"iif.html#format_ne/3"},{"type":"function","title":"iif.ife/2","doc":"Return Value if first argument is one of: [] , false , undefined . Otherwise return the value of the first argument.","ref":"iif.html#ife/2"},{"type":"function","title":"iif.ife/3","doc":"Return Empty if first argument is one of: [] , false , undefined . Otherwise, if NotEmpty is fun() , evaluate it, or if it's fun(Arg) evaluate it with Value argument.","ref":"iif.html#ife/3"},{"type":"function","title":"iif.ifne/2","doc":"Return Value if first argument is not one of: [] , false , undefined . Otherwise, if Value is fun() , evaluate it, or if it's fun(Arg) evaluate it with Test argument. If not empty","ref":"iif.html#ifne/2"},{"type":"function","title":"iif.ifne/3","doc":"Return NotEmpty if first argument is not one of: [] , false , undefined . Otherwise, if NotEmpty is fun() , evaluate it, or if it's fun(Arg) evaluate it with Value argument.","ref":"iif.html#ifne/3"},{"type":"function","title":"iif.iif/3","doc":"Return True if first argument is true or return False if the first argument is one of: [] , false , undefined .","ref":"iif.html#iif/3"},{"type":"function","title":"iif.iif/4","doc":"Return True if first two arguments match","ref":"iif.html#iif/4"},{"type":"function","title":"iif.nvl/2","doc":"Alias for ife/2","ref":"iif.html#nvl/2"},{"type":"function","title":"iif.parse_transform/2","doc":"Parse transform to be used by providing {parse_transform, iif} option. Opts are compiler options passed from command line. E.g.: erlc - Diif_ast . . . -&gt; Opts = [ { d , iif_ast } | _ ] erlc - Diif_orig . . . -&gt; Opts = [ { d , iif_orig } | _ ] erlc - Diif_src . . . -&gt; Opts = [ { d , iif_src } | _ ]","ref":"iif.html#parse_transform/2"},{"type":"module","title":"listcomp","doc":"Erlang map-reduce parse transform This transform introduces two modifications of the list comprehension syntax that allow to perform a fold and mapfold on a list. Indexed List Comprehension This extension of a list comprehension, passes an additional argument to the left hand side of the comprehension, which is the index of the current item in the list: [ io : format ( &quot;Rec# ~w : ~p \\n&quot; , [ I , N ] ) || I , N &lt;- L ] ^ ^ ` ` ` The index is defined by the a variable listed after the ` || &#39; operator . This is equivalent to the following : ` ` ` lists : mapfoldl ( fun ( N , I ) -&gt; io : format ( &quot;Rec# ~w : ~p \\n&quot; , [ I , N ] ) , I + 1 end , 1 , L ) ` ` ` == = Fold Comprehension == = To invoke the fold comprehension transform include the initial state assignment into a comprehension that returns a non - tuple expression : ` ` ` [ S + N || S = 1 , N &lt;- L ] . ^ ^ ^ ^ ^ ^ ^ ^ In this example the S variable gets assigned the initial state 1 , and the S+N expression represents the body of the fold function that is passed the iteration variable N and the state variable S : lists : foldl ( fun ( N , S ) -&gt; S + N end , 1 , L ) . Fold comprehension can be combined with the indexed list comprehension: [ running_sum ( I , N , S + N ) || I , S = 5 , N &lt;- L ] . running_sum ( I , N , RunningSum ) -&gt; io : format ( &quot;Rec# ~w : ~p ( ~w )\\n&quot; , [ I , N , RunningSum ] ) , S . In this case the definition of the indexed fold comprehension would be transformed to: element ( 2 , lists : foldl ( fun ( I , { N , S } ) -&gt; running_sum ( I , N , S ) , { N + 1 , S + N } end , { 1 , 5 } , L ) ) , Compilation When using this as a parse transform, include the {parse_transform,listcomp} compiler option. For debugging the AST of the resulting transform, pass the following options to the erlc compiler: -Dlistcomp_orig - print the original AST before the transform -Dlistcomp_ast - print the transformed AST -Dlistcomp_src - print the resulting source code after the transform","ref":"listcomp.html"},{"type":"function","title":"listcomp.foldl/3","doc":"Fold over a list by additionally passing the list's current item number to the folding fun. This function is similar to lists:foldl/3, except that the fun takes the extra second integer argument that represents the sequential number of the item from the list.","ref":"listcomp.html#foldl/3"},{"type":"function","title":"listcomp.foldr/3","doc":"Fold over a list by additionally passing the list's current item number to the folding fun. This function is similar to lists:foldr/3, except that the fun takes the extra second integer argument that represents the sequential number of the item from the list.","ref":"listcomp.html#foldr/3"},{"type":"function","title":"listcomp.parse_transform/2","doc":"parse_transform entry point","ref":"listcomp.html#parse_transform/2"},{"type":"module","title":"str","doc":"Parse transform that implements str/2 Use {parse_transform,str} compiler's option to use this transform. str ( Fmt , Args ) -&gt; lists : flatten ( io_lib : format ( Fmt , Args ) ) throw ( Fmt , Args ) -&gt; erlang : throw ( lists : flatten ( io_lib : format ( Fmt , Args ) ) error ( Fmt , Args ) -&gt; erlang : error ( lists : flatten ( io_lib : format ( Fmt , Args ) ) i2l ( Int ) -&gt; integer_to_list ( Int ) % Enabled with compiled with % the `{d,str_i2l}&#39; option b2l ( Bin ) -&gt; binary_to_list ( Bin ) % Enabled with compiled with % the `{d,str_b2l}&#39; option str ( Term ) -&gt; str : str ( Term )","ref":"str.html"},{"type":"function","title":"str.get_float_fmt/0","doc":"Get custom float format from the process dictionary","ref":"str.html#get_float_fmt/0"},{"type":"function","title":"str.parse_transform/2","doc":"Parse transform to be used by providing {parse_transform, str} option.","ref":"str.html#parse_transform/2"},{"type":"function","title":"str.reset_float_fmt/0","doc":"Erase custom float format from the process dictionary","ref":"str.html#reset_float_fmt/0"},{"type":"function","title":"str.set_float_fmt/1","doc":"Store custom float format in the process dictionary Return previously stored format. Also see float_to_list/2 http://erlang.org/doc/man/erlang.html#float_to_list-2","ref":"str.html#set_float_fmt/1"},{"type":"function","title":"str.str/1","doc":"Stringify an argument","ref":"str.html#str/1"},{"type":"function","title":"str.str/2","doc":"","ref":"str.html#str/2"},{"type":"module","title":"tmp","doc":"Erlang pipeline parse transform This transform implements a parser syntax extension that enables application of cascading function calls using the / operator. In the LHS / RHS / ... Last. notation, the result of evaluation of the LHS expression is passed as an argument to the RHS expression. This process continues until the Last expression is evaluated. The head element of the pipeline must be either a term to which the arithmetic division /` operator cannot apply (i.e. not integers, floats, functions), or if you need to pass integer(s) or float(s), wrap them in a list brackets. This transfor is inspired by the similar functionality in Linux (i.e. `| pipe) and Elixir ( |&gt; pipe). When using this as a parse transform, include the {parse_transform,erlpipe} compiler option. The following examples illustrate the work of the transform, in which: test1 ( A ) -&gt; [ A ] / fun1 / mod : fun2 / fun3 . test2 ( A , B ) -&gt; [ A , B ] / fun4 / fun5 ( ) / io : format ( &quot; ~p \\n&quot; , [ _ ] ) . will be transformed to: test1 ( A ) -&gt; fun3 ( mod : fun2 ( fun1 ( A ) ) ) . test2 ( A , B ) -&gt; io : format ( &quot; ~p \\n&quot; , [ fun5 ( fun4 ( A , B ) ) ] ) . Similarly to Elixir, a special tap/2 function is implemented, which passes the given argument to an anonymous function, returning the argument itself. The following: f ( A ) -&gt; A + 1 . . . . test_tap ( ) -&gt; [ 10 ] / tap ( f ) / tap ( fun f / 1 ) / tap ( fun ( I ) -&gt; I + 1 end ) . is equivalent to: . . . test_tap ( ) -&gt; begin f ( 10 ) , begin f ( 10 ) , begin ( fun ( I ) -&gt; I end ) ( 10 ) 10 end end end . For debugging the AST of the resulting transform, pass the following options to the erlc compiler: -Derlpipe_orig - print the original AST before the transform -Derlpipe_ast - print the transformed AST -Derlpipe_src - print the resulting source code after the transform","ref":"tmp.html"},{"type":"function","title":"tmp.parse_transform/2","doc":"parse_transform entry point","ref":"tmp.html#parse_transform/2"},{"type":"extras","title":"Overview","doc":"Author : Serge Aleynikov &lt;saleyn(at)gmail.com&gt; License : MIT License This library includes useful parse transforms including Elixir-like pipeline operator for cascading function calls.","ref":"readme.html"},{"type":"extras","title":"Overview - Content","doc":"Module Description defarg Support default argument values in Erlang functions erlpipe Elixir-like pipe operator for Erlang listcomp Fold Comprehension and Indexed List Comprehension iif Ternary if function including iif/3 , iif/4 , nvl/2 , nvl/3 parse transforms str Stringification functions including str/1 , str/2 , and throw/2 parse transforms","ref":"readme.html#content"},{"type":"extras","title":"Overview - defarg: Support default argument values in Erlang functions","doc":"Presently the Erlang syntax doesn't allow function arguments to have default parameters. Consequently a developer needs to replicate the function definition multiple times passing constant defaults to some parameters of functions. This parse transform addresses this shortcoming by extending the syntax of function definitions at the top level in a module to have a default expression such that for A / Default argument the Default will be used if the function is called in code without that argument. Though it might seem more intuitive for programmers coming from other languages to use the assignment operator = for defining default arguments, using that operator would change the current meaning of pattern matching of arguments in function calls (i.e. test(A=10) is presently a valid expression). Therefore we chose the / operator for declaring default arguments because it has no valid meaning when applied in declaration of function arguments, and presently without the defarg transform, using this operator (e.g. test(A / 10) -&gt; ... ) would result in a syntax error detected by the compiler. - export ( [ t / 2 ] ) . test ( A / 10 , B / 20 ) -&gt; A + B . The code above is transformed to: - export ( [ t / 2 ] ) . - export ( [ t / 0 , t / 1 ] ) . test ( ) -&gt; test ( 10 ) ; test ( A ) -&gt; test ( A , 20 ) ; test ( A , B ) -&gt; A + B . The arguments with default values must be at the end of the argument list: test ( A , B , C / 1 ) -&gt; %% This is valid . . . test ( A / 1 , B , C ) -&gt; %% This is invalid . . . NOTE: The default arguments should be constant expressions. Function calls in default arguments are not supported! test ( A / erlang : timestamp ( ) ) -&gt; %% !!! Bad syntax . . .","ref":"readme.html#defarg-support-default-argument-values-in-erlang-functions"},{"type":"extras","title":"Overview - erlpipe: Erlang Pipe Operator","doc":"Inspired by the Elixir's |&gt; pipeline operator. This transform makes code with cascading function calls much more readable by using the / as the pipeline operator. In the LHS / RHS / ... Last. notation, the result of evaluation of the LHS expression is passed as an argument to the RHS expression. This process continues until the Last expression is evaluated. The head element of the pipeline must be either a term to which the arithmetic division / operator cannot apply (i.e. not integers, floats, variables, functions), or if you need to pass an integer, float, variable, or a result of a function call, wrap it in a list brackets. It transforms code from: print ( L ) when is_list ( L ) -&gt; [ 3 , L ] %% Multiple items in a list become arguments to the first function / lists : split %% In Module:Function calls parenthesis are optional / element ( 1 , _ ) %% &#39;_&#39; is the placeholder for the return value of a previous call / binary_to_list / io : format ( &quot; ~s \\n&quot; , [ _ ] ) . test1 ( Arg1 , Arg2 , Arg3 ) -&gt; [ Arg1 , Arg2 ] %% Arguments must be enclosed in `[...]` / fun1 %% In function calls parenthesis are optional / mod : fun2 / fun3 ( ) / fun4 ( Arg3 , _ ) %% &#39;_&#39; is the placeholder for the return value of a previous call / fun ff / 1 %% Inplace function references are supported / fun erlang : length / 1 %% Inplace Mod:Fun/Arity function references are supported / fun ( I ) -&gt; I end %% This lambda will be evaluated as: (fun(I) -&gt; I end)(_) / io_lib : format ( &quot; ~p \\n&quot; , [ _ ] ) / fun6 ( [ 1 , 2 , 3 ] , _ , other_param ) / fun7 . test2 ( ) -&gt; % Result = Argument / Function 3 = abc / atom_to_list / length , %% Atoms can be passed to &#39;/&#39; as is 3 = &quot;abc&quot; / length , %% Strings can be passed to &#39;/&#39; as is &quot;abc&quot; = &lt;&lt; &quot;abc&quot; &gt;&gt; / binary_to_list , %% Binaries can be passed to &#39;/&#39; as is &quot;1,2,3&quot; = { $1 , $2 , $3 } / tuple_to_list %% Tuples can be passed to &#39;/&#39; as is / [ [ I ] || I &lt;- _ ] %% The &#39;_&#39; placeholder is replaced by the return of tuple_to_list/1 / string : join ( &quot;,&quot; ) , %% Here a call to string:join/2 is made &quot;1&quot; = [ min ( 1 , 2 ) ] / integer_to_list , %% Function calls, integer and float value &quot;1&quot; = [ 1 ] / integer_to_list , %% arguments must be enclosed in a list. &quot;1.0&quot; = [ 1.0 ] / float_to_list ( [ { decimals , 1 } ] ) , &quot;abc\\n&quot; = &quot;abc&quot; / ( _ ++ &quot;\\n&quot; ) , %% Can use operators on the right hand side 2.0 = 4.0 / max ( 1.0 , 2.0 ) , %% Expressions with lhs floats are unmodified 2 = 4 / max ( 1 , 2 ) . %% Expressions with lhs integers are unmodified test3 ( ) -&gt; A = 10 , B = 5 , 2 = A / B , %% LHS variables (e.g. A) are not affected by the transform 2.0 = 10 / 5 , %% Arithmetic division for integers, floats, variables is unmodified 2.0 = A / 5 , %% (ditto) 5 = max ( A , B ) / 2 . %% Use of division on LHS function calls is unaffected by the transform to the following equivalent: test1 ( Arg1 , Arg2 , Arg3 ) -&gt; fun7 ( fun6 ( [ 1 , 2 , 3 ] , io_lib : format ( &quot; ~p \\n&quot; , [ ( fun ( I ) -&gt; I end ) ( erlang : length ( ff ( fun4 ( Arg3 , fun3 ( mod2 : fun2 ( fun1 ( Arg1 , Arg2 ) ) ) ) ) ) ) ] ) , other_param ) ) . print ( L ) when is_list ( L ) -&gt; io : format ( &quot; ~s \\n&quot; , [ binary_to_list ( element ( 1 , lists : split ( 3 , L ) ) ) ] ) . test2 ( ) -&gt; 3 = length ( atom_to_list ( abc ) ) , 3 = length ( &quot;abc&quot; ) , &quot;abc&quot; = binary_to_list ( &lt;&lt; &quot;abc&quot; &gt;&gt; ) , &quot;1,2,3&quot; = string : join ( [ [ I ] || I &lt;- tuple_to_list ( { $1 , $2 , $3 } ) ] , &quot;,&quot; ) , &quot;1&quot; = integer_to_list ( min ( 1 , 2 ) ) , &quot;1&quot; = integer_to_list ( 1 ) , &quot;1.0&quot; = float_to_list ( 1.0 , [ { decimals , 1 } ] ) , &quot;abc\\n&quot; = &quot;abc&quot; ++ &quot;\\n&quot; , 2.0 = 4.0 / max ( 1.0 , 2.0 ) , 2 = 4 / max ( 1 , 2 ) . Similarly to Elixir, a special tap/2 function is implemented, which passes the given argument to an anonymous function, returning the argument itself. The following: f ( A ) -&gt; A + 1 . . . . test_tap ( ) -&gt; [ 10 ] / tap ( f ) / tap ( fun f / 1 ) / tap ( fun ( I ) -&gt; I + 1 end ) . is equivalent to: . . . test_tap ( ) -&gt; begin f ( 10 ) , begin f ( 10 ) , begin ( fun ( I ) -&gt; I + 1 end ) ( 10 ) , 10 end end end . Some attempts to tackle this pipeline transform have been done by other developers: https://github.com/fenollp/fancyflow https://github.com/stolen/pipeline https://github.com/oltarasenko/epipe https://github.com/clanchun/epipe https://github.com/pouriya/pipeline Yet, we subjectively believe that the choice of syntax in this implementation of transform is more succinct and elegant, and doesn't attempt to modify the meaning of the / operator for arithmetic LHS types (i.e. integers, floats, variables, and function calls). Why didn't we use |&gt; operator instead of / to make it equivalent to Elixir? Parse transforms are applied only after the Erlang source code gets parsed to the AST representation, which must be in valid Erlang syntax. The |&gt; operator is not known to the Erlang parser, and therefore, using it would result in the compile-time error. We had to select an operator that the Erlang parser would be happy with, and / was our choice because visually it resembles the pipe | character more than the other operators.","ref":"readme.html#erlpipe-erlang-pipe-operator"},{"type":"extras","title":"Overview - listcomp: Fold and Indexed List Comprehensions","doc":"Indexed List Comprehension Occasionally the body of a list comprehension needs to know the index of the current item in the fold. Consider this example: [ { 1 , 10 } , { 2 , 20 } ] = element ( 1 , lists : foldmapl ( fun ( I , N ) -&gt; { { N , I } , N + 1 } end , 1 , [ 10 , 20 ] ) ) . Here the N variable is tracking the index of the current item I in the list. While the same result in this specific case can be achieved with lists:zip(lists:seq(1,2), [10,20]) , in a more general case, there is no way to have an item counter propagated with the current list comprehension syntax. The Indexed List Comprehension accomplishes just that through the use of an unassigned variable immediately to the right of the || operator: [ { Idx , I } || Idx , I &lt;- L ] . % ^^^ % | % +--- This variable becomes the index counter Example: [ { 1 , 10 } , { 2 , 20 } ] = [ { Idx , I } || Idx , I &lt;- [ 10 , 20 ] ] . Fold Comprehension To invoke the fold comprehension transform include the initial state assignment into a list comprehension: [ S + I || S = 1 , I &lt;- L ] . % ^^^ ^^^^^ % | | % | +--- State variable bound to the initial value % +----------- The body of the foldl function In this example the S variable gets assigned the initial state 1 , and the S+I expression represents the body of the fold function that is passed the iteration variable I and the state variable S : lists : foldl ( fun ( I , S ) -&gt; S + I end , 1 , L ) . A fold comprehension can be combined with the indexed list comprehension by using this syntax: [ do ( Idx , S + I ) || Idx , S = 10 , I &lt;- L ] . % ^^^^^^^^^^^^ ^^^ ^^^^^^ % | | | % | | +--- State variable bound to the initial value (e.g. 10) % | +--------- The index variable bound to the initial value of 1 % +--------------------- The body of the foldl function can use Idx and S This code is transformed to: element ( 2 , lists : foldl ( fun ( I , { Idx , S } ) -&gt; { Idx + 1 , do ( Idx , S + I ) } end , { 1 , 10 } , L ) ) . Example: 33 = [ S + Idx * I || Idx , S = 1 , I &lt;- [ 10 , 20 ] ] , 30 = [ print ( Idx , I , S ) || Idx , S = 0 , I &lt;- [ 10 , 20 ] ] . % Prints: % Item#1 running sum: 10 % Item#2 running sum: 30 print ( Idx , I , S ) -&gt; Res = S + I , io : format ( &quot;Item# ~w running sum: ~w \\n&quot; , [ Idx , Res ] ) , Res .","ref":"readme.html#listcomp-fold-and-indexed-list-comprehensions"},{"type":"extras","title":"Overview - iif: Ternary and quaternary if","doc":"This transform improves the code readability for cases that involve simple conditional if/then/else tests in the form iif(Condition, Then, Else) . Since this is a parse transform, the Then and Else expressions are evaluated only if the Condition evaluates to true or false respectively. E.g.: iif ( tuple_size ( T ) == 3 , good , bad ) . %% Ternary if iif ( some_fun ( A ) , match , ok , error ) . %% Quaternary if nvl ( L , undefined ) . nvl ( L , nil , hd ( L ) ) are transformed to: case tuple_size ( T ) == 3 of true -&gt; good ; _ -&gt; bad end . case some_fun ( A ) of match -&gt; ok ; nomatch -&gt; error end . case L of [ ] -&gt; undefined ; false -&gt; undefined ; undefined -&gt; undefined ; _ -&gt; L end . case L of [ ] -&gt; nil ; false -&gt; nil ; undefined -&gt; nil ; _ -&gt; hd ( L ) end .","ref":"readme.html#iif-ternary-and-quaternary-if"},{"type":"extras","title":"Overview - str: String transforms","doc":"This module implements a transform to stringify an Erlang term. str(Term) is equivalent to lists:flatten(io_lib:format(&quot;~p&quot;, [Term])) for terms that are not integers, floats, atoms, binaries and lists. Integers, atoms, and binaries are converted to string using *_to_list/1 functions. Floats are converted using float_to_list/2 where the second argument is controled by str:set_float_fmt/1 and str:reset_float_fmt/0 calls. Lists are converted to string using lists:flatten(io_lib:format(&quot;~s&quot;, [Term])) and if that fails, then using lists:flatten(io_lib:format(&quot;~p&quot;, [Term])) format. str(Fmt, Args) is equivalent to lists:flatten(io_lib:format(Fmt, Args)) . throw(Fmt,Args) is equivalent to throw(lists:flatten(io_lib:format(Fmt, Args))) . Two other shorthand transforms are optionally supported: b2l(Binary) is equivalent to binary_to_list(Binary) (enabled by giving {d,str_b2l} ) compilation option. i2l(Integer) is equivalent to integer_to_list(Binary) (enabled by giving {d,str_i2l} ) compilation option. E.g.: erlc + debug_info - Dstr_b2l - Dstr_i2l + &#39;{parse_transform, str}&#39; - o ebin your_module . erl","ref":"readme.html#str-string-transforms"},{"type":"extras","title":"Overview - Dowloading","doc":"Github Hex.pm","ref":"readme.html#dowloading"},{"type":"extras","title":"Overview - Building and Using","doc":"$ make To use the transforms, compile your module with the +'{parse_transform, Module}' command-line option, or include -compile({parse_transform, Module}). in your source code, where Module is one of the transform modules implemented in this project. To use all transforms implemented by the etran application, compile your module with this command-line option: +'{parse_transform, etran}' . erlc + debug_info + &#39;{parse_transform, etran}&#39; - o ebin your_module . erl If you are using rebar3 to build your project, then add to rebar.config : { deps , [ { etran , &quot;0.5.1&quot; } ] } . { erl_opts , [ debug_info , { parse_transform , etran } ] } .","ref":"readme.html#building-and-using"},{"type":"extras","title":"License","doc":"MIT License Copyright (c) 2021 Serge Aleynikov Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}]